@@ -21,22 +21,18 @@
 
 """Generic io_object handler, io specific check methods."""
 
-__all__ = [ "PY2",
-            "IO_Object", "IO_Object_ContentHandler", "IO_Object_XMLGenerator",
+__all__ = [ "IO_Object", "IO_Object_ContentHandler", "IO_Object_XMLGenerator",
             "check_port", "check_tcpudp", "check_protocol", "check_address" ]
 
 import xml.sax as sax
 import xml.sax.saxutils as saxutils
 import copy
-import sys
+from collections import OrderedDict
 
 from firewall import functions
-from firewall.functions import b2u
 from firewall import errors
 from firewall.errors import FirewallError
 
-PY2 = sys.version < '3'
-
 class IO_Object(object):
     """ Abstract IO_Object as base for icmptype, service and zone """
 
@@ -59,21 +55,44 @@ class IO_Object(object):
             ret.append(copy.deepcopy(getattr(self, x[0])))
         return tuple(ret)
 
-    def import_config(self, conf):
-        self.check_config(conf)
+    def export_config_dict(self):
+        conf = {}
+        type_formats = dict([(x[0], x[1]) for x in self.IMPORT_EXPORT_STRUCTURE])
+        for key in type_formats:
+            if getattr(self, key) or isinstance(getattr(self, key), bool):
+                conf[key] = copy.deepcopy(getattr(self, key))
+        return conf
+
+    def import_config(self, conf, all_io_objects):
+        self.check_config(conf, all_io_objects)
         for i,(element,dummy) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
             if isinstance(conf[i], list):
                 # remove duplicates without changing the order
                 _conf = [ ]
+                _set = set()
                 for x in conf[i]:
-                    if x not in _conf:
+                    if x not in _set:
                         _conf.append(x)
+                        _set.add(x)
+                del _set
                 setattr(self, element, copy.deepcopy(_conf))
             else:
                 setattr(self, element, copy.deepcopy(conf[i]))
 
+    def import_config_dict(self, conf, all_io_objects):
+        self.check_config_dict(conf, all_io_objects)
+
+        for key in conf:
+            if not hasattr(self, key):
+                raise FirewallError(errors.UNKNOWN_ERROR, "Internal error. '{}' is not a valid attribute".format(key))
+            if isinstance(conf[key], list):
+                # maintain list order while removing duplicates
+                setattr(self, key, list(OrderedDict.fromkeys(copy.deepcopy(conf[key]))))
+            else:
+                setattr(self, key, copy.deepcopy(conf[key]))
+
     def check_name(self, name):
-        if type(name) != type(""):
+        if not isinstance(name, str):
             raise FirewallError(errors.INVALID_TYPE,
                                 "'%s' not of type %s, but %s" % (name, type(""),
                                                                  type(name)))
@@ -85,22 +104,31 @@ class IO_Object(object):
                     errors.INVALID_NAME,
                     "'%s' is not allowed in '%s'" % ((char, name)))
 
-    def check_config(self, conf):
+    def check_config(self, conf, all_io_objects={}):
         if len(conf) != len(self.IMPORT_EXPORT_STRUCTURE):
             raise FirewallError(
                 errors.INVALID_TYPE,
                 "structure size mismatch %d != %d" % \
                 (len(conf), len(self.IMPORT_EXPORT_STRUCTURE)))
-        for i,(element,value) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
-            self._check_config_structure(conf[i], value)
-            self._check_config(conf[i], element)
+        conf_dict = {}
+        for i,(x,y) in enumerate(self.IMPORT_EXPORT_STRUCTURE):
+            conf_dict[x] = conf[i]
+        self.check_config_dict(conf_dict, all_io_objects)
 
-    def _check_config(self, dummy1, dummy2):
+    def check_config_dict(self, conf, all_io_objects):
+        type_formats = dict([(x[0], x[1]) for x in self.IMPORT_EXPORT_STRUCTURE])
+        for key in conf:
+            if key not in [x for (x,y) in self.IMPORT_EXPORT_STRUCTURE]:
+                raise FirewallError(errors.INVALID_OPTION, "option '{}' is not valid".format(key))
+            self._check_config_structure(conf[key], type_formats[key])
+            self._check_config(conf[key], key, conf, all_io_objects)
+
+    def _check_config(self, dummy1, dummy2, dummy3, dummy4):
         # to be overloaded by sub classes
         return
 
     def _check_config_structure(self, conf, structure):
-        if not type(conf) == type(structure):
+        if not isinstance(conf, type(structure)):
             raise FirewallError(errors.INVALID_TYPE,
                                 "'%s' not of type %s, but %s" % \
                                 (conf, type(structure), type(conf)))
@@ -122,14 +150,14 @@ class IO_Object(object):
             # only one key value pair in structure
             (skey, svalue) = list(structure.items())[0]
             for (key, value) in conf.items():
-                if type(key) != type(skey):
+                if not isinstance(key, type(skey)):
                     raise FirewallError(errors.INVALID_TYPE,
                                         "'%s' not of type %s, but %s" % (\
-                            key, type(skey), type(key)))
-                if type(value) != type(svalue):
+                                            key, type(skey), type(key)))
+                if not isinstance(value, type(svalue)):
                     raise FirewallError(errors.INVALID_TYPE,
                                         "'%s' not of type %s, but %s" % (\
-                            value, type(svalue), type(value)))
+                                            value, type(svalue), type(value)))
 
     # check required elements and attributes and also optional attributes
     def parser_check_element_attrs(self, name, attrs):
@@ -224,54 +252,13 @@ class IO_Object_XMLGenerator(saxutils.XMLGenerator):
         self._pending_start_element = False
         self._short_empty_elements = False
 
-    def startElement(self, name, attrs):
-        """ saxutils.XMLGenerator.startElement() expects name and attrs to be
-            unicode and bad things happen if any of them is (utf-8) encoded.
-            We override the method here to sanitize this case.
-            Can be removed once we drop Python2 support.
-        """
-        if PY2:
-            attrs = { b2u(name):b2u(value) for name, value in attrs.items() }
-        saxutils.XMLGenerator.startElement(self, name, attrs)
-
     def simpleElement(self, name, attrs):
         """ slightly modified startElement()
         """
-        if PY2:
-            self._write(u'<' + b2u(name))
-            for (name, value) in attrs.items():
-                self._write(u' %s=%s' % (b2u(name),
-                                         saxutils.quoteattr(b2u(value))))
-            self._write(u'/>')
-        else:
-            self._write('<' + name)
-            for (name, value) in attrs.items():
-                self._write(' %s=%s' % (name, saxutils.quoteattr(value)))
-            self._write('/>')
-
-    def endElement(self, name):
-        """ saxutils.XMLGenerator.endElement() expects name to be
-            unicode and bad things happen if it's (utf-8) encoded.
-            We override the method here to sanitize this case.
-            Can be removed once we drop Python2 support.
-        """
-        saxutils.XMLGenerator.endElement(self, b2u(name))
-
-    def characters(self, content):
-        """ saxutils.XMLGenerator.characters() expects content to be
-            unicode and bad things happen if it's (utf-8) encoded.
-            We override the method here to sanitize this case.
-            Can be removed once we drop Python2 support.
-        """
-        saxutils.XMLGenerator.characters(self, b2u(content))
-
-    def ignorableWhitespace(self, content):
-        """ saxutils.XMLGenerator.ignorableWhitespace() expects content to be
-            unicode and bad things happen if it's (utf-8) encoded.
-            We override the method here to sanitize this case.
-            Can be removed once we drop Python2 support.
-        """
-        saxutils.XMLGenerator.ignorableWhitespace(self, b2u(content))
+        self._write('<' + name)
+        for (name, value) in attrs.items():
+            self._write(' %s=%s' % (name, saxutils.quoteattr(value)))
+        self._write('/>')
 
 def check_port(port):
     port_range = functions.getPortRange(port)
@@ -289,9 +276,10 @@ def check_port(port):
                             "'%s' is invalid port range" % port)
 
 def check_tcpudp(protocol):
-    if protocol not in [ "tcp", "udp" ]:
+    if protocol not in [ "tcp", "udp", "sctp", "dccp" ]:
         raise FirewallError(errors.INVALID_PROTOCOL,
-                            "'%s' not from {'tcp'|'udp'}" % protocol)
+                            "'%s' not from {'tcp'|'udp'|'sctp'|'dccp'}" % \
+                            protocol)
 
 def check_protocol(protocol):
     if not functions.checkProtocol(protocol):
@@ -301,4 +289,3 @@ def check_address(ipv, addr):
     if not functions.check_address(ipv, addr):
         raise FirewallError(errors.INVALID_ADDR,
                             "'%s' is not valid %s address" % (addr, ipv))
-
