@@ -26,9 +26,8 @@ import os
 import io
 import shutil
 
-from firewall.config import ETC_FIREWALLD
-from firewall.functions import u2b_if_py2
-from firewall.core.io.io_object import PY2, IO_Object, \
+from firewall import config
+from firewall.core.io.io_object import IO_Object, \
     IO_Object_ContentHandler, IO_Object_XMLGenerator, check_port, \
     check_tcpudp, check_protocol, check_address
 from firewall.core.logger import log
@@ -37,22 +36,23 @@ from firewall.errors import FirewallError
 
 class Service(IO_Object):
     IMPORT_EXPORT_STRUCTURE = (
-        ( "version",  "" ),                   # s
-        ( "short", "" ),                      # s
-        ( "description", "" ),                # s
-        ( "ports", [ ( "", "" ), ], ),        # a(ss)
-        ( "modules", [ "", ], ),              # as
-        ( "destination", { "": "", }, ),      # a{ss}
-        ( "protocols", [ "", ], ),            # as
-        ( "source_ports", [ ( "", "" ), ], ), # a(ss)
-        )
-    DBUS_SIGNATURE = '(sssa(ss)asa{ss}asa(ss))'
+        ( "version",  "" ),
+        ( "short", "" ),
+        ( "description", "" ),
+        ( "ports", [ ( "", "" ), ], ),
+        ( "modules", [ "", ], ),
+        ( "destination", { "": "", }, ),
+        ( "protocols", [ "", ], ),
+        ( "source_ports", [ ( "", "" ), ], ),
+        ( "includes", [ "" ], ),
+        ( "helpers", [ "", ], ),
+    )
     ADDITIONAL_ALNUM_CHARS = [ "_", "-" ]
     PARSER_REQUIRED_ELEMENT_ATTRS = {
         "short": None,
         "description": None,
         "service": None,
-        }
+    }
     PARSER_OPTIONAL_ELEMENT_ATTRS = {
         "service": [ "name", "version" ],
         "port": [ "port", "protocol" ],
@@ -60,7 +60,9 @@ class Service(IO_Object):
         "module": [ "name" ],
         "destination": [ "ipv4", "ipv6" ],
         "source-port": [ "port", "protocol" ],
-        }
+        "include": [ "service" ],
+        "helper": [ "name" ],
+    }
 
     def __init__(self):
         super(Service, self).__init__()
@@ -72,6 +74,8 @@ class Service(IO_Object):
         self.modules = [ ]
         self.destination = { }
         self.source_ports = [ ]
+        self.includes = [ ]
+        self.helpers = [ ]
 
     def cleanup(self):
         self.version = ""
@@ -82,22 +86,10 @@ class Service(IO_Object):
         del self.modules[:]
         self.destination.clear()
         del self.source_ports[:]
+        del self.includes[:]
+        del self.helpers[:]
 
-    def encode_strings(self):
-        """ HACK. I haven't been able to make sax parser return
-            strings encoded (because of python 2) instead of in unicode.
-            Get rid of it once we throw out python 2 support."""
-        self.version = u2b_if_py2(self.version)
-        self.short = u2b_if_py2(self.short)
-        self.description = u2b_if_py2(self.description)
-        self.ports = [(u2b_if_py2(po),u2b_if_py2(pr)) for (po,pr) in self.ports]
-        self.modules = [u2b_if_py2(m) for m in self.modules]
-        self.destination = {u2b_if_py2(k):u2b_if_py2(v) for k,v in self.destination.items()}
-        self.protocols = [u2b_if_py2(pr) for pr in self.protocols]
-        self.source_ports = [(u2b_if_py2(po),u2b_if_py2(pr)) for (po,pr)
-                             in self.source_ports]
-
-    def _check_config(self, config, item):
+    def _check_config(self, config, item, all_config, all_io_objects):
         if item == "ports":
             for port in config:
                 if port[0] != "":
@@ -126,11 +118,20 @@ class Service(IO_Object):
 
         elif item == "modules":
             for module in config:
-                if not module.startswith("nf_conntrack_"):
-                    raise FirewallError(errors.INVALID_MODULE, module)
-                elif len(module.replace("nf_conntrack_", "")) < 1:
+                if module.startswith("nf_conntrack_"):
+                    module = module.replace("nf_conntrack_", "")
+                    if "_" in module:
+                        module = module.replace("_", "-")
+                if len(module) < 2:
                     raise FirewallError(errors.INVALID_MODULE, module)
 
+        elif item == "includes":
+            for include in config:
+                if include not in all_io_objects["services"]:
+                    raise FirewallError(errors.INVALID_SERVICE,
+                            "Service '{}': Included service '{}' not found.".format(
+                                self.name, include))
+
 # PARSER
 
 class service_ContentHandler(IO_Object_ContentHandler):
@@ -190,15 +191,28 @@ class service_ContentHandler(IO_Object_ContentHandler):
                     else:
                         self.item.destination[x] = attrs[x]
         elif name == "module":
-            if attrs["name"].startswith("nf_conntrack_") and \
-               len(attrs["name"].replace("nf_conntrack_", "")) > 0:
-                if attrs["name"] not in self.item.modules:
-                    self.item.modules.append(attrs["name"])
-                else:
-                    log.warning("Module '%s' already set, ignoring.",
-                                attrs["name"])
+            module = attrs["name"]
+            if module.startswith("nf_conntrack_"):
+                module = module.replace("nf_conntrack_", "")
+                if "_" in module:
+                    module = module.replace("_", "-")
+            if module not in self.item.modules:
+                self.item.modules.append(module)
             else:
-                log.warning("Invalid module '%s'", attrs["name"])
+                log.warning("Module '%s' already set, ignoring.",
+                            module)
+        elif name == "include":
+            if attrs["service"] not in self.item.includes:
+                self.item.includes.append(attrs["service"])
+            else:
+                log.warning("Include '%s' already set, ignoring.",
+                            attrs["service"])
+        elif name == "helper":
+            if attrs["name"] not in self.item.helpers:
+                self.item.helpers.append(attrs["name"])
+            else:
+                log.warning("Helper '%s' already set, ignoring.",
+                            attrs["name"])
 
 
 def service_reader(filename, path):
@@ -210,23 +224,23 @@ def service_reader(filename, path):
     service.check_name(service.name)
     service.filename = filename
     service.path = path
-    service.builtin = False if path.startswith(ETC_FIREWALLD) else True
+    service.builtin = False if path.startswith(config.ETC_FIREWALLD) else True
     service.default = service.builtin
     handler = service_ContentHandler(service)
     parser = sax.make_parser()
     parser.setContentHandler(handler)
     name = "%s/%s" % (path, filename)
-    with open(name, "r") as f:
+    with open(name, "rb") as f:
+        source = sax.InputSource(None)
+        source.setByteStream(f)
         try:
-            parser.parse(f)
+            parser.parse(source)
         except sax.SAXParseException as msg:
             raise FirewallError(errors.INVALID_SERVICE,
                                 "not a valid service file: %s" % \
                                 msg.getException())
     del handler
     del parser
-    if PY2:
-        service.encode_strings()
     return service
 
 def service_writer(service, path=None):
@@ -244,9 +258,9 @@ def service_writer(service, path=None):
             log.error("Backup of file '%s' failed: %s", name, msg)
 
     dirpath = os.path.dirname(name)
-    if dirpath.startswith(ETC_FIREWALLD) and not os.path.exists(dirpath):
-        if not os.path.exists(ETC_FIREWALLD):
-            os.mkdir(ETC_FIREWALLD, 0o750)
+    if dirpath.startswith(config.ETC_FIREWALLD) and not os.path.exists(dirpath):
+        if not os.path.exists(config.ETC_FIREWALLD):
+            os.mkdir(config.ETC_FIREWALLD, 0o750)
         os.mkdir(dirpath, 0o750)
 
     f = io.open(name, mode='wt', encoding='UTF-8')
@@ -307,6 +321,18 @@ def service_writer(service, path=None):
         handler.simpleElement("destination", service.destination)
         handler.ignorableWhitespace("\n")
 
+    # includes
+    for include in service.includes:
+        handler.ignorableWhitespace("  ")
+        handler.simpleElement("include", { "service": include })
+        handler.ignorableWhitespace("\n")
+
+    # helpers
+    for helper in service.helpers:
+        handler.ignorableWhitespace("  ")
+        handler.simpleElement("helper", { "name": helper })
+        handler.ignorableWhitespace("\n")
+
     # end service element
     handler.endElement('service')
     handler.ignorableWhitespace("\n")
