@@ -43,6 +43,10 @@ RT_TO_PERM_FAILED   =   31
 IPSET_WITH_TIMEOUT  =   32
 BUILTIN_IPSET       =   33
 ALREADY_SET         =   34
+MISSING_IMPORT      =   35
+DBUS_ERROR          =   36
+BUILTIN_HELPER      =   37
+NOT_APPLIED         =   38
 
 INVALID_ACTION      =  100
 INVALID_SERVICE     =  101
@@ -82,6 +86,12 @@ INVALID_MAC         =  134
 INVALID_IPSET       =  135
 INVALID_ENTRY       =  136
 INVALID_OPTION      =  137
+INVALID_HELPER      =  138
+INVALID_PRIORITY    =  139
+INVALID_POLICY      =  140
+INVALID_LOG_PREFIX  =  141
+INVALID_NFLOG_GROUP =  142
+INVALID_NFLOG_QUEUE =  143
 
 MISSING_TABLE       =  200
 MISSING_CHAIN       =  201
@@ -92,6 +102,7 @@ MISSING_NAME        =  205
 MISSING_SETTING     =  206
 MISSING_FAMILY      =  207
 
+RUNNING_BUT_FAILED  =  251
 NOT_RUNNING         =  252
 NOT_AUTHORIZED      =  253
 UNKNOWN_ERROR       =  254
@@ -101,12 +112,6 @@ import sys
 class FirewallError(Exception):
     def __init__(self, code, msg=None):
         self.code = code
-        if msg is not None:
-            # escape msg if needed
-            try:
-                x = str(msg)
-            except UnicodeEncodeError:
-                msg = unicode(msg).encode("unicode_escape")
         self.msg = msg
 
     def __repr__(self):
@@ -118,7 +123,7 @@ class FirewallError(Exception):
         return self.errors[self.code]
 
     def get_code(msg):
-        if ":" in msg:
+        if msg is not None and ":" in msg:
             idx = msg.index(":")
             ecode = msg[:idx]
         else:
