@@ -19,16 +19,35 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+__all__ = [ "getPortID", "getPortRange", "portStr", "getServiceName",
+            "checkIP", "checkIP6", "checkIPnMask", "checkIP6nMask",
+            "checkProtocol", "checkInterface", "checkUINT16", "checkUINT32",
+            "firewalld_is_active", "tempFile", "readfile", "writefile",
+            "enable_ip_forwarding", "check_port", "check_address",
+            "check_single_address", "check_mac", "uniqify", "ppid_of_pid",
+            "max_zone_name_len", "checkUser", "checkUid", "checkCommand",
+            "checkContext", "joinArgs", "splitArgs",
+            "max_policy_name_len", "checkTcpMssClamp",
+            "stripNonPrintableCharacters"]
+
 import socket
+import os
 import os.path
-import shlex, pipes
+import shlex
+import pipes
 import string
-import sys
 import tempfile
 from firewall.core.logger import log
-from firewall.config import FIREWALLD_TEMPDIR
+from firewall.config import FIREWALLD_TEMPDIR, FIREWALLD_PIDFILE
 
-PY2 = sys.version < '3'
+NOPRINT_TRANS_TABLE = {
+    # Limit to C0 and C1 code points. Building entries for all unicode code
+    # points requires too much memory.
+    # C0 = [0, 31]
+    # C1 = [127, 159]
+    #
+    i: None for i in range(0, 160) if not (i > 31 and i < 127)
+}
 
 def getPortID(port):
     """ Check and Get port id from port string or port id using socket.getservbyname
@@ -36,22 +55,22 @@ def getPortID(port):
     @param port port string or port id
     @return Port id if valid, -1 if port can not be found and -2 if port is too big
     """
-    
+
     if isinstance(port, int):
-        id = port
+        _id = port
     else:
         if port:
             port = port.strip()
         try:
-            id = int(port)
+            _id = int(port)
         except ValueError:
             try:
-                id = socket.getservbyname(port)
-            except:
+                _id = socket.getservbyname(port)
+            except socket.error:
                 return -1
-    if id > 65535:
+    if _id > 65535:
         return -2
-    return id
+    return _id
 
 def getPortRange(ports):
     """ Get port range for port range string or single port id
@@ -60,13 +79,32 @@ def getPortRange(ports):
     @return Array containing start and end port id for a valid range or -1 if port can not be found and -2 if port is too big for integer input or -1 for invalid ranges or None if the range is ambiguous.
     """
 
-    if isinstance(ports, int):
-        id = getPortID(ports)
-        if id >= 0:
-            return (id,)
-        return id
+    # (port, port)  or [port, port] case
+    if isinstance(ports, tuple) or isinstance(ports, list):
+        return ports
+
+    # "<port-id>" case
+    if isinstance(ports, int) or ports.isdigit():
+        id1 = getPortID(ports)
+        if id1 >= 0:
+            return (id1,)
+        return id1
 
     splits = ports.split("-")
+
+    # "<port-id>-<port-id>" case
+    if len(splits) == 2 and splits[0].isdigit() and splits[1].isdigit():
+        id1 = getPortID(splits[0])
+        id2 = getPortID(splits[1])
+        if id1 >= 0 and id2 >= 0:
+            if id1 < id2:
+                return (id1, id2)
+            elif id1 > id2:
+                return (id2, id1)
+            else: # ids are the same
+                return (id1,)
+
+    # everything else "<port-str>[-<port-str>]"
     matched = [ ]
     for i in range(len(splits), 0, -1):
         id1 = getPortID("-".join(splits[:i]))
@@ -93,8 +131,8 @@ def getPortRange(ports):
     return matched[0]
 
 def portStr(port, delimiter=":"):
-    """ Create port and port range string 
-    
+    """ Create port and port range string
+
     @param port port or port range int or [int, int]
     @param delimiter of the output string for port ranges, default ':'
     @return Port or port range string, empty string if port isn't specified, None if port or port range is not valid
@@ -102,13 +140,112 @@ def portStr(port, delimiter=":"):
     if port == "":
         return ""
 
-    range = getPortRange(port)
-    if isinstance(range, int) and range < 0:
+    _range = getPortRange(port)
+    if isinstance(_range, int) and _range < 0:
         return None
-    elif len(range) == 1:
-        return "%s" % range
+    elif len(_range) == 1:
+        return "%s" % _range
     else:
-        return "%s%s%s" % (range[0], delimiter, range[1])
+        return "%s%s%s" % (_range[0], delimiter, _range[1])
+
+def portInPortRange(port, range):
+    _port = getPortRange(port)
+    _range = getPortRange(range)
+
+    if len(_port) == 1:
+        if len(_range) == 1:
+            return getPortID(_port[0]) == getPortID(_range[0])
+        if len(_range) == 2 and \
+           getPortID(_port[0]) >= getPortID(_range[0]) and getPortID(_port[0]) <= getPortID(_range[1]):
+            return True
+    elif len(_port) == 2:
+        if len(_range) == 2 and \
+           getPortID(_port[0]) >= getPortID(_range[0]) and getPortID(_port[0]) <= getPortID(_range[1]) and \
+           getPortID(_port[1]) >= getPortID(_range[0]) and getPortID(_port[1]) <= getPortID(_range[1]):
+            return True
+
+    return False
+
+def coalescePortRange(new_range, ranges):
+    """ Coalesce a port range with existing list of port ranges
+
+        @param new_range tuple/list/string
+        @param ranges list of tuple/list/string
+        @return tuple of (list of ranges added after coalescing, list of removed original ranges)
+    """
+
+    coalesced_range = getPortRange(new_range)
+    # normalize singleton ranges, e.g. (x,) --> (x,x)
+    if len(coalesced_range) == 1:
+        coalesced_range = (coalesced_range[0], coalesced_range[0])
+    _ranges = map(getPortRange, ranges)
+    _ranges = sorted(map(lambda x: (x[0],x[0]) if len(x) == 1 else x, _ranges), key=lambda x: x[0])
+
+    removed_ranges = []
+    for range in _ranges:
+        if coalesced_range[0] <= range[0] and coalesced_range[1] >= range[1]:
+            # new range covers this
+            removed_ranges.append(range)
+        elif coalesced_range[0] <= range[0] and coalesced_range[1] <  range[1] and \
+                                                coalesced_range[1] >= range[0]:
+            # expand beginning of range
+            removed_ranges.append(range)
+            coalesced_range = (coalesced_range[0], range[1])
+        elif coalesced_range[0] >  range[0] and coalesced_range[1] >= range[1] and \
+                                                coalesced_range[0] <= range[1]:
+            # expand end of range
+            removed_ranges.append(range)
+            coalesced_range = (range[0], coalesced_range[1])
+
+    # normalize singleton ranges, e.g. (x,x) --> (x,)
+    removed_ranges = list(map(lambda x: (x[0],) if x[0] == x[1] else x, removed_ranges))
+    if coalesced_range[0] == coalesced_range[1]:
+        coalesced_range = (coalesced_range[0],)
+
+    return ([coalesced_range], removed_ranges)
+
+def breakPortRange(remove_range, ranges):
+    """ break a port range from existing list of port ranges
+
+        @param remove_range tuple/list/string
+        @param ranges list of tuple/list/string
+        @return tuple of (list of ranges added after breaking up, list of removed original ranges)
+    """
+
+    remove_range = getPortRange(remove_range)
+    # normalize singleton ranges, e.g. (x,) --> (x,x)
+    if len(remove_range) == 1:
+        remove_range = (remove_range[0], remove_range[0])
+    _ranges = map(getPortRange, ranges)
+    _ranges = sorted(map(lambda x: (x[0],x[0]) if len(x) == 1 else x, _ranges), key=lambda x: x[0])
+
+    removed_ranges = []
+    added_ranges = []
+    for range in _ranges:
+        if remove_range[0] <= range[0] and remove_range[1] >= range[1]:
+            # remove entire range
+            removed_ranges.append(range)
+        elif remove_range[0] <= range[0] and remove_range[1] <  range[1] and \
+                                             remove_range[1] >= range[0]:
+            # remove from beginning of range
+            removed_ranges.append(range)
+            added_ranges.append((remove_range[1] + 1, range[1]))
+        elif remove_range[0] >  range[0] and remove_range[1] >= range[1] and \
+                                             remove_range[0] <= range[1]:
+            # remove from end of range
+            removed_ranges.append(range)
+            added_ranges.append((range[0], remove_range[0] - 1))
+        elif remove_range[0] > range[0] and remove_range[1] < range[1]:
+            # remove inside range
+            removed_ranges.append(range)
+            added_ranges.append((range[0], remove_range[0] - 1))
+            added_ranges.append((remove_range[1] + 1, range[1]))
+
+    # normalize singleton ranges, e.g. (x,x) --> (x,)
+    removed_ranges = list(map(lambda x: (x[0],) if x[0] == x[1] else x, removed_ranges))
+    added_ranges = list(map(lambda x: (x[0],) if x[0] == x[1] else x, added_ranges))
+
+    return (added_ranges, removed_ranges)
 
 def getServiceName(port, proto):
     """ Check and Get service name from port and proto string combination using socket.getservbyport
@@ -120,13 +257,13 @@ def getServiceName(port, proto):
 
     try:
         name = socket.getservbyport(int(port), proto)
-    except:
+    except socket.error:
         return None
     return name
 
 def checkIP(ip):
     """ Check IPv4 address.
-    
+
     @param ip address string
     @return True if address is valid, else False
     """
@@ -137,31 +274,42 @@ def checkIP(ip):
         return False
     return True
 
+def normalizeIP6(ip):
+    """ Normalize the IPv6 address
+
+    This is mostly about converting URL-like IPv6 address to normal ones.
+    e.g. [1234::4321] --> 1234:4321
+    """
+    return ip.strip("[]")
+
 def checkIP6(ip):
     """ Check IPv6 address.
-    
+
     @param ip address string
     @return True if address is valid, else False
     """
 
     try:
-        socket.inet_pton(socket.AF_INET6, ip)
+        socket.inet_pton(socket.AF_INET6, normalizeIP6(ip))
     except socket.error:
         return False
     return True
 
 def checkIPnMask(ip):
-    if "/" in ip:
-        addr = ip[:ip.index("/")]
-        mask = ip[ip.index("/")+1:]
+    index = ip.find("/")
+    if index != -1:
+        addr = ip[:index]
+        mask = ip[index + 1:]
+        if not addr or not mask:
+            return False
     else:
         addr = ip
         mask = None
     if not checkIP(addr):
         return False
     if mask:
-        if "." in mask and checkIP(addr):
-            return False
+        if "." in mask:
+            return checkIP(mask)
         else:
             try:
                 i = int(mask)
@@ -171,10 +319,16 @@ def checkIPnMask(ip):
                 return False
     return True
 
+def stripNonPrintableCharacters(rule_str):
+    return rule_str.translate(NOPRINT_TRANS_TABLE)
+
 def checkIP6nMask(ip):
-    if "/" in ip:
-        addr = ip[:ip.index("/")]
-        mask = ip[ip.index("/")+1:]
+    index = ip.find("/")
+    if index != -1:
+        addr = ip[:index]
+        mask = ip[index + 1:]
+        if not addr or not mask:
+            return False
     else:
         addr = ip
         mask = None
@@ -197,7 +351,7 @@ def checkProtocol(protocol):
         # string
         try:
             socket.getprotobyname(protocol)
-        except:
+        except socket.error:
             return False
     else:
         if i < 0 or i > 255:
@@ -205,6 +359,17 @@ def checkProtocol(protocol):
 
     return True
 
+def checkTcpMssClamp(tcp_mss_clamp_value):
+    if tcp_mss_clamp_value:
+        if tcp_mss_clamp_value.isdigit():
+            if int(tcp_mss_clamp_value) < 536:
+                return False
+        elif tcp_mss_clamp_value == "None":
+            return True
+        elif tcp_mss_clamp_value != "pmtu":
+            return False
+    return True
+
 def checkInterface(iface):
     """ Check interface string
 
@@ -224,10 +389,20 @@ def checkInterface(iface):
     #    return False
     return True
 
+def checkUINT16(val):
+    try:
+        x = int(val, 0)
+    except ValueError:
+        return False
+    else:
+        if x >= 0 and x <= 65535:
+            return True
+    return False
+
 def checkUINT32(val):
     try:
         x = int(val, 0)
-    except:
+    except ValueError:
         return False
     else:
         if x >= 0 and x <= 4294967295:
@@ -240,13 +415,13 @@ def firewalld_is_active():
     @return True if there is a firewalld pid file and the pid is used by firewalld
     """
 
-    if not os.path.exists("/var/run/firewalld.pid"):
+    if not os.path.exists(FIREWALLD_PIDFILE):
         return False
 
     try:
-        with open("/var/run/firewalld.pid", "r") as fd:
+        with open(FIREWALLD_PIDFILE, "r") as fd:
             pid = fd.readline()
-    except:
+    except Exception:
         return False
 
     if not os.path.exists("/proc/%s" % pid):
@@ -255,7 +430,7 @@ def firewalld_is_active():
     try:
         with open("/proc/%s/cmdline" % pid, "r") as fd:
             cmdline = fd.readline()
-    except:
+    except Exception:
         return False
 
     if "firewalld" in cmdline:
@@ -265,15 +440,16 @@ def firewalld_is_active():
 
 def tempFile():
     try:
+        if not os.path.exists(FIREWALLD_TEMPDIR):
+            os.mkdir(FIREWALLD_TEMPDIR, 0o750)
+
         return tempfile.NamedTemporaryFile(mode='wt', prefix="temp.",
                                            dir=FIREWALLD_TEMPDIR, delete=False)
     except Exception as msg:
         log.error("Failed to create temporary file: %s" % msg)
         raise
-    return None
 
 def readfile(filename):
-    i = 1
     try:
         with open(filename, "r") as f:
             return f.readlines()
@@ -297,17 +473,20 @@ def enable_ip_forwarding(ipv):
         return writefile("/proc/sys/net/ipv6/conf/all/forwarding", "1\n")
     return False
 
+def get_nf_conntrack_short_name(module):
+    return module.replace("_","-").replace("nf-conntrack-", "")
+
 def check_port(port):
-    range = getPortRange(port)
-    if range == -2 or range == -1 or range is None or \
-            (len(range) == 2 and range[0] >= range[1]):
-        if range == -2:
+    _range = getPortRange(port)
+    if _range == -2 or _range == -1 or _range is None or \
+            (len(_range) == 2 and _range[0] >= _range[1]):
+        if _range == -2:
             log.debug2("'%s': port > 65535" % port)
-        elif range == -1:
+        elif _range == -1:
             log.debug2("'%s': port is invalid" % port)
-        elif range is None:
+        elif _range is None:
             log.debug2("'%s': port is ambiguous" % port)
-        elif len(range) == 2 and range[0] >= range[1]:
+        elif len(_range) == 2 and _range[0] >= _range[1]:
             log.debug2("'%s': range start >= end" % port)
         return False
     return True
@@ -340,10 +519,10 @@ def check_mac(mac):
         return True
     return False
 
-def uniqify(input):
+def uniqify(_list):
     # removes duplicates from list, whilst preserving order
     output = []
-    for x in input:
+    for x in _list:
         if x not in output:
             output.append(x)
     return output
@@ -354,18 +533,29 @@ def ppid_of_pid(pid):
         f = os.popen("ps -o ppid -h -p %d 2>/dev/null" % pid)
         pid = int(f.readlines()[0].strip())
         f.close()
-    except:
+    except Exception:
         return None
     return pid
 
+def max_policy_name_len():
+    """
+    iptables limits length of chain to (currently) 28 chars.
+    The longest chain we create is POST_<policy>_allow,
+    which leaves 28 - 11 = 17 chars for <policy>.
+    """
+    from firewall.core.ipXtables import POLICY_CHAIN_PREFIX
+    from firewall.core.base import SHORTCUTS
+    longest_shortcut = max(map(len, SHORTCUTS.values()))
+    return 28 - (longest_shortcut + len(POLICY_CHAIN_PREFIX) + len("_allow"))
+
 def max_zone_name_len():
     """
     Netfilter limits length of chain to (currently) 28 chars.
-    The longest chain we create is FWDI_<zone>_allow,
+    The longest chain we create is POST_<zone>_allow,
     which leaves 28 - 11 = 17 chars for <zone>.
     """
     from firewall.core.base import SHORTCUTS
-    longest_shortcut = max(map (len, SHORTCUTS.values()))
+    longest_shortcut = max(map(len, SHORTCUTS.values()))
     return 28 - (longest_shortcut + len("__allow"))
 
 def checkUser(user):
@@ -379,7 +569,7 @@ def checkUser(user):
     return True
 
 def checkUid(uid):
-    if type(uid) == str:
+    if isinstance(uid, str):
         try:
             uid = int(uid)
         except ValueError:
@@ -422,29 +612,5 @@ def joinArgs(args):
     else:
         return " ".join(pipes.quote(a) for a in args)
 
-def splitArgs(string):
-    if PY2 and isinstance(string, unicode):
-        # Python2's shlex doesn't like unicode
-        string = u2b(string)
-        splits = shlex.split(string)
-        return map (b2u, splits)
-    else:
-        return shlex.split(string)
-
-def b2u(string):
-    """ bytes to unicode """
-    if isinstance(string, bytes):
-        return string.decode('UTF-8', 'replace')
-    return string
-
-def u2b(string):
-    """ unicode to bytes """
-    if not isinstance(string, bytes):
-        return string.encode('UTF-8', 'replace')
-    return string
-
-def u2b_if_py2(string):
-    """ unicode to bytes only if Python 2"""
-    if PY2 and isinstance(string, unicode):
-            return string.encode('UTF-8', 'replace')
-    return string
+def splitArgs(_string):
+    return shlex.split(_string)
