@@ -18,28 +18,26 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# force use of pygobject3 in python-slip
-from gi.repository import GObject
-import sys
-sys.modules['gobject'] = GObject
-
 import dbus
 import dbus.service
-import slip.dbus
-import slip.dbus.service
 
 from firewall import config
 from firewall.dbus_utils import dbus_to_python, \
     dbus_introspection_prepare_properties, \
     dbus_introspection_add_properties
 from firewall.core.io.zone import Zone
+from firewall.core.fw_ifcfg import ifcfg_set_zone_of_interface
 from firewall.core.base import DEFAULT_ZONE_TARGET
 from firewall.core.rich import Rich_Rule
 from firewall.core.logger import log
+from firewall.server.dbus import DbusServiceObject
 from firewall.server.decorators import handle_exceptions, \
-    dbus_handle_exceptions, dbus_service_method
+    dbus_handle_exceptions, dbus_service_method, \
+    dbus_polkit_require_auth
 from firewall import errors
 from firewall.errors import FirewallError
+from firewall.functions import portStr, portInPortRange, coalescePortRange, \
+                               breakPortRange
 
 ############################################################################
 #
@@ -47,7 +45,7 @@ from firewall.errors import FirewallError
 #
 ############################################################################
 
-class FirewallDConfigZone(slip.dbus.service.Object):
+class FirewallDConfigZone(DbusServiceObject):
     """FirewallD main class"""
 
     persistent = True
@@ -94,9 +92,8 @@ class FirewallDConfigZone(slip.dbus.service.Object):
             return dbus.Boolean(self.obj.builtin)
         else:
             raise dbus.exceptions.DBusException(
-                "org.freedesktop.DBus.Error.AccessDenied: "
-                "Property '%s' isn't exported (or may not exist)" % \
-                property_name)
+                "org.freedesktop.DBus.Error.InvalidArgs: "
+                "Property '%s' does not exist" % property_name)
 
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ss',
                          out_signature='v')
@@ -111,7 +108,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         if interface_name != config.dbus.DBUS_INTERFACE_CONFIG_ZONE:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.UnknownInterface: "
-                "FirewallD does not implement %s" % interface_name)
+                "Interface '%s' does not exist" % interface_name)
 
         return self._get_property(property_name)
 
@@ -125,14 +122,14 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         if interface_name != config.dbus.DBUS_INTERFACE_CONFIG_ZONE:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.UnknownInterface: "
-                "FirewallD does not implement %s" % interface_name)
+                "Interface '%s' does not exist" % interface_name)
 
         ret = { }
         for x in [ "name", "filename", "path", "default", "builtin" ]:
             ret[x] = self._get_property(x)
         return dbus.Dictionary(ret, signature="sv")
 
-    @slip.dbus.polkit.require_auth(config.dbus.PK_ACTION_CONFIG)
+    @dbus_polkit_require_auth(config.dbus.PK_ACTION_CONFIG)
     @dbus_service_method(dbus.PROPERTIES_IFACE, in_signature='ssv')
     @dbus_handle_exceptions
     def Set(self, interface_name, property_name, new_value, sender=None):
@@ -146,11 +143,11 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         if interface_name != config.dbus.DBUS_INTERFACE_CONFIG_ZONE:
             raise dbus.exceptions.DBusException(
                 "org.freedesktop.DBus.Error.UnknownInterface: "
-                "FirewallD does not implement %s" % interface_name)
+                "Interface '%s' does not exist" % interface_name)
 
         raise dbus.exceptions.DBusException(
-            "org.freedesktop.DBus.Error.AccessDenied: "
-            "Property '%s' is not settable" % property_name)
+            "org.freedesktop.DBus.Error.PropertyReadOnly: "
+            "Property '%s' is read-only" % property_name)
 
     @dbus.service.signal(dbus.PROPERTIES_IFACE, signature='sa{sv}as')
     def PropertiesChanged(self, interface_name, changed_properties,
@@ -161,7 +158,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         log.debug1("%s.PropertiesChanged('%s', '%s', '%s')", self._log_prefix,
                    interface_name, changed_properties, invalidated_properties)
 
-    @slip.dbus.polkit.require_auth(config.dbus.PK_ACTION_INFO)
+    @dbus_polkit_require_auth(config.dbus.PK_ACTION_INFO)
     @dbus_service_method(dbus.INTROSPECTABLE_IFACE, out_signature='s')
     @dbus_handle_exceptions
     def Introspect(self, sender=None): # pylint: disable=W0613
@@ -176,7 +173,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
     # S E T T I N G S
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
-                         out_signature=Zone.DBUS_SIGNATURE)
+                         out_signature="(sssbsasa(ss)asba(ssss)asasasasa(ss)b)")
     @dbus_handle_exceptions
     def getSettings(self, sender=None): # pylint: disable=W0613
         """get settings for zone
@@ -190,16 +187,34 @@ class FirewallDConfigZone(slip.dbus.service.Object):
             settings = tuple(_settings)
         return settings
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
+                         out_signature="a{sv}")
+    @dbus_handle_exceptions
+    def getSettings2(self, sender=None):
+        """get settings for zone
+        """
+        log.debug1("%s.getSettings2()", self._log_prefix)
+        settings = self.config.get_zone_config_dict(self.obj)
+        if settings["target"] == DEFAULT_ZONE_TARGET:
+            settings["target"] = "default"
+        return settings
+
     def _checkDuplicateInterfacesSources(self, settings):
         """Assignment of interfaces/sources to zones is different from other
            zone settings in the sense that particular interface/zone can be
            part of only one zone. So make sure added interfaces/sources have
            not already been bound to another zone."""
-        old_settings = self.config.get_zone_config(self.obj)
-        idx_i = Zone.index_of("interfaces")
-        idx_s = Zone.index_of("sources")
-        added_ifaces = set(settings[idx_i]) - set(old_settings[idx_i])
-        added_sources = set(settings[idx_s]) - set(old_settings[idx_s])
+        old_settings = self.config.get_zone_config_dict(self.obj)
+        old_ifaces = set(old_settings["interfaces"]) if "interfaces" in old_settings else set()
+        old_sources = set(old_settings["sources"]) if "sources" in old_settings else set()
+        if isinstance(settings, tuple):
+            added_ifaces = set(settings[Zone.index_of("interfaces")]) - old_ifaces
+            added_sources = set(settings[Zone.index_of("sources")]) - old_sources
+        else: # dict
+            new_ifaces = set(settings["interfaces"]) if "interfaces" in settings else set()
+            new_sources = set(settings["sources"]) if "sources" in settings else set()
+            added_ifaces = new_ifaces - old_ifaces
+            added_sources = new_sources - old_sources
 
         for iface in added_ifaces:
             if self.parent.getZoneOfInterface(iface):
@@ -209,7 +224,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                 raise FirewallError(errors.ZONE_CONFLICT, source) # or move to new zone ?
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
-                         in_signature=Zone.DBUS_SIGNATURE)
+                         in_signature="(sssbsasa(ss)asba(ssss)asasasasa(ss)b)")
     @dbus_handle_exceptions
     def update(self, settings, sender=None):
         """update settings for zone
@@ -226,6 +241,21 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         self.obj = self.config.set_zone_config(self.obj, settings)
         self.Updated(self.obj.name)
 
+    @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
+                         in_signature="a{sv}")
+    @dbus_handle_exceptions
+    def update2(self, settings, sender=None):
+        """update settings for zone
+        """
+        settings = dbus_to_python(settings)
+        log.debug1("%s.update2('...')", self._log_prefix)
+        self.parent.accessCheck(sender)
+        if "target" in settings and settings["target"] == "default":
+            settings["target"] = DEFAULT_ZONE_TARGET
+        self._checkDuplicateInterfacesSources(settings)
+        self.obj = self.config.set_zone_config_dict(self.obj, settings)
+        self.Updated(self.obj.name)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE)
     @dbus_handle_exceptions
     def loadDefaults(self, sender=None):
@@ -368,7 +398,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getServices(self, sender=None): # pylint: disable=W0613
         log.debug1("%s.getServices()", self._log_prefix)
-        return sorted(self.getSettings()[5])
+        return self.getSettings()[5]
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
                          in_signature='as')
@@ -454,10 +484,16 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) in settings[6]:
-            raise FirewallError(errors.ALREADY_ENABLED,
-                                "%s:%s" % (port, protocol))
-        settings[6].append((port,protocol))
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[6]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                raise FirewallError(errors.ALREADY_ENABLED,
+                                    "%s:%s" % (port, protocol))
+        added_ranges, removed_ranges = coalescePortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[6].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[6].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -470,9 +506,17 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) not in settings[6]:
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[6]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                break
+        else:
             raise FirewallError(errors.NOT_ENABLED, "%s:%s" % (port, protocol))
-        settings[6].remove((port,protocol))
+        added_ranges, removed_ranges = breakPortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[6].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[6].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -483,7 +527,11 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         protocol = dbus_to_python(protocol, str)
         log.debug1("%s.queryPort('%s', '%s')", self._log_prefix, port,
                    protocol)
-        return (port,protocol) in self.getSettings()[6]
+        for (_port, _protocol) in self.getSettings()[6]:
+            if portInPortRange(port, _port) and protocol == _protocol:
+                return True
+
+        return False
 
     # protocol
 
@@ -578,10 +626,16 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) in settings[14]:
-            raise FirewallError(errors.ALREADY_ENABLED,
-                                "%s:%s" % (port, protocol))
-        settings[14].append((port,protocol))
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[14]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                raise FirewallError(errors.ALREADY_ENABLED,
+                                    "%s:%s" % (port, protocol))
+        added_ranges, removed_ranges = coalescePortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[14].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[14].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -594,9 +648,17 @@ class FirewallDConfigZone(slip.dbus.service.Object):
                    protocol)
         self.parent.accessCheck(sender)
         settings = list(self.getSettings())
-        if (port,protocol) not in settings[14]:
+        existing_port_ids = list(filter(lambda x: x[1] == protocol, settings[14]))
+        for port_id in existing_port_ids:
+            if portInPortRange(port, port_id[0]):
+                break
+        else:
             raise FirewallError(errors.NOT_ENABLED, "%s:%s" % (port, protocol))
-        settings[14].remove((port,protocol))
+        added_ranges, removed_ranges = breakPortRange(port, [_port for (_port, _protocol) in existing_port_ids])
+        for range in removed_ranges:
+            settings[14].remove((portStr(range, "-"), protocol))
+        for range in added_ranges:
+            settings[14].append((portStr(range, "-"), protocol))
         self.update(settings)
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
@@ -607,7 +669,11 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         protocol = dbus_to_python(protocol, str)
         log.debug1("%s.querySourcePort('%s', '%s')", self._log_prefix, port,
                    protocol)
-        return (port,protocol) in self.getSettings()[14]
+        for (_port, _protocol) in self.getSettings()[14]:
+            if portInPortRange(port, _port) and protocol == _protocol:
+                return True
+
+        return False
 
     # icmp block
 
@@ -616,7 +682,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getIcmpBlocks(self, sender=None): # pylint: disable=W0613
         log.debug1("%s.getIcmpBlocks()", self._log_prefix)
-        return sorted(self.getSettings()[7])
+        return self.getSettings()[7]
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
                          in_signature='as')
@@ -852,7 +918,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getInterfaces(self, sender=None): # pylint: disable=W0613
         log.debug1("%s.getInterfaces()", self._log_prefix)
-        return sorted(self.getSettings()[10])
+        return self.getSettings()[10]
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
                          in_signature='as')
@@ -879,6 +945,8 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         settings[10].append(interface)
         self.update(settings)
 
+        ifcfg_set_zone_of_interface(self.obj.name, interface)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
                          in_signature='s')
     @dbus_handle_exceptions
@@ -892,6 +960,8 @@ class FirewallDConfigZone(slip.dbus.service.Object):
         settings[10].remove(interface)
         self.update(settings)
 
+        ifcfg_set_zone_of_interface("", interface)
+
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
                          in_signature='s',
                          out_signature='b')
@@ -908,7 +978,7 @@ class FirewallDConfigZone(slip.dbus.service.Object):
     @dbus_handle_exceptions
     def getSources(self, sender=None): # pylint: disable=W0613
         log.debug1("%s.getSources()", self._log_prefix)
-        return sorted(self.getSettings()[11])
+        return self.getSettings()[11]
 
     @dbus_service_method(config.dbus.DBUS_INTERFACE_CONFIG_ZONE,
                          in_signature='as')
